/**
 * Example of how to call the TCB DistributedMeasurement service.
 * 
 * This client demo is responsible for starting a distributed measurement
 * that is ended by server demo. Only server demo presents the measurement
 * results.
 * 
 * @author Paulo Sousa
 */

#include "stdafx.h"
//include sockets header file
#include <winsock.h>
//include tcb services library header file
#include "..\..\..\tcb_services_library\tcb_services_library.h"


//TODO: replace this values according to your situation
#define MY_IP "192.168.0.1"
#define SERVER_IP "192.168.0.2"
#define SERVER_PORT 40000


int main(int argc, char* argv[]) {

	//open_session
	printf("Opening TCB session\n");
	int error = open_tcb_session();
	if (error!=0) {
		printf("Error opening session=%d\n",error);
		getchar();
		return -1;
	}

	//create a TCB socket
	printf("Creating TCB socket\n");
	error = tcb_socket(AF_INET);
	if (error!=0) {
		printf("Error creating TCB socket: %d\n", error);
		getchar();
		return -1;
	}

	//bind TCB socket
	printf("Binding TCB socket\n");
	struct sockaddr_in sa;
	sa.sin_family=AF_INET;
	sa.sin_addr.s_addr= inet_addr(MY_IP);
	sa.sin_port=htons(0);
	error=tcb_bind((struct sockaddr*)&sa, sizeof(sa));
	if (error!=0) {
		printf("Error binding TCB socket: %d\n", error);
		getchar();
		return -1;
	}


	//start a distributed measurement
	printf("Starting a dist. measurement\n");
	//always reserve TCB_DATA_SIZE bytes in the end of the buffer
	char buf[50+TCB_DATA_SIZE];
	//msg to be sent
	strcpy(buf, "Hi server!");
	struct sockaddr_in to;
	to.sin_family = AF_INET;
	to.sin_addr.s_addr = inet_addr(SERVER_IP);
	to.sin_port = htons(SERVER_PORT);
		
	int nbytes = tcb_sendto(buf, 50+TCB_DATA_SIZE, NULL, (struct sockaddr*)&to, 
						    sizeof(to), get_tcb_timestamp());
		
	if (nbytes<0) {
		printf("Error starting dist. measurement: %d\n", nbytes);
	}
	else printf("%d bytes sent\n", nbytes);


	//close_session
	printf("Closing session\n");
	error=close_tcb_session();
	if (error!=0) {
		printf("Error closing session=%d\n",error);
		getchar();
		return -1;
	}
	printf("Close ok!\n");

	//wait for user to press a key
	printf("Press a key to close window...\n");
	getchar();
	return 0;
}
