/**
 * TCB for Windows CE 3.0
 *
 * This header file defines the TCB configuration parameters.
 *
 * @author Paulo Sousa
 * @date 18/Nov/2002
 */

#ifndef CONFIG_PARAMETERS_TCB
#define CONFIG_PARAMETERS_TCB


//Number of machines running TCB
//Note: When changing this value, check your configuration in machines_description.h
#define NUMBER_OF_MACHINES 3

//Indicates if we are using acks or omission degree for data messages
#define USE_ACK

#ifdef USE_ACK

	//maximum number of retransmissions
	#define MAX_RETRANSMITS 5
	//maximum time (ms) to wait for acks
	#define ACK_TIMEOUT 5

#else

	//Data messages omission degree
	#define DATA_OMISSION_DEGREE 3

#endif

//Token messages omission degree
#define TOKEN_OMISSION_DEGREE 0

//802.11 MTU 
#define MTU 2000

//Size of socket receive buffer
#define SOCK_RCV_BUF_SIZE 4096

//local clock drift rate
#define CLOCK_DRIFT_RATE 0.005


/* TODO: descomentar aps testes com packet size fixo

//Maximum packet size according to MTU,SOCK_RCV_BUF_SIZE,
//NUMBER_OF_MACHINES and DATA_OMISSION_DEGREE
#if MTU<(SOCK_RCV_BUF_SIZE/NUMBER_OF_MACHINES)

 #define MAX_PACKET_SIZE MTU/(DATA_OMISSION_DEGREE+1)

#else

 #define MAX_PACKET_SIZE (SOCK_RCV_BUF_SIZE/NUMBER_OF_MACHINES)/(DATA_OMISSION_DEGREE+1)

#endif

*/

#define MAX_PACKET_SIZE 500



/*Type of Broadcast*/
//Using real broacast only when NUMBER_OF_MACHINES>1
#if NUMBER_OF_MACHINES>1

	//for real broadcast, define the broadcast address to use
	#define REAL_BROADCAST "169.254.0.255"

#else

	//emulated broadcast= send unicast packet to all machines
	#define EMUL_BROADCAST

#endif

#ifdef USE_ACK

	//Maximum time that a machine waits for a token/data msg (milliseconds)
	#if NUMBER_OF_MACHINES>1
		////maximum delivery time of a token msg (milliseconds)
		#define MAXIMUM_TOKEN_DELIVERY_DELAY 50
		//maximum time until token arrives to predecessor + maximum transmission time of a data msg
		#define MSG_TIMEOUT ((MAXIMUM_TOKEN_DELIVERY_DELAY*(NUMBER_OF_MACHINES-1))+(ACK_TIMEOUT*(MAX_RETRANSMITS+1)+MAXIMUM_DELIVERY_DELAY))
	#else
		#define MSG_TIMEOUT (ACK_TIMEOUT*(MAX_RETRANSMITS+1)+MAXIMUM_DELIVERY_DELAY)
	#endif

#else

	//Maximum time that a machine waits for a token/data msg (milliseconds)
	#define MSG_TIMEOUT 184

#endif


//Period of NetworkThread (milliseconds) (only active when NUMBER_OF_MACHINES==1)
#define NET_PERIOD 10


#ifdef USE_ACK

	//Number of tokens that are necessary to be received between each send
	#define TOKENS_PER_SEND 1

#else

	//Number of tokens that are necessary to be received between each send
	#define TOKENS_PER_SEND 4

#endif

//Period of TCB SendThread (milliseconds) (worst case).
//In the worst case, a token is received after MSG_TIMEOUT*(NUMBER_OF_MACHINES-1) ms.
#define SEND_PERIOD (MSG_TIMEOUT*(NUMBER_OF_MACHINES-1)*TOKENS_PER_SEND)

//Period of TCB RecvThread (milliseconds)
#define RECV_PERIOD 1

//send queue size
#define SIZE_SEND_QUEUE 1
//NOTE: Data msgs are produced according to token speed, i.e., a msg is produced
//per TOKENS_PER_SEND received. So, at any time, there is only one msg in the queue.

//receive queue size
#define SIZE_RECEIVE_QUEUE (NUMBER_OF_MACHINES*10)
//NOTE: In normal circunstances, the maximum size is NUMBER_OF_MACHINES.
//Nevertheless we multiply by a factor of 10 as a safety margin.

//maximum number of simultaneous local measures
#define MAX_LOCAL_MEASURES 10

//maximum number of simultaneous timely executions
#define MAX_EXEC 10

//maximum number of simultaneous local timing failure detections
#define MAX_LOCAL_FD 10

//minimum delivery time of a data msg (milliseconds)
#define MINIMUM_DELIVERY_DELAY 0

//maximum delivery time of a data msg (milliseconds)
#define MAXIMUM_DELIVERY_DELAY 330

//TFDmax for distributed timing failure detection with FD in sender
#define TFDMAX_WITH_SENDER_FD ((2*SEND_PERIOD + MAXIMUM_DELIVERY_DELAY + RECV_PERIOD)*2)

//TFDmax for distributed timing failure detection with FD in recipients
#define TFDMAX_WITH_REMOTE_FD (2*SEND_PERIOD + MAXIMUM_DELIVERY_DELAY + RECV_PERIOD)

#endif
